@@ -1,22 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0
 /*
  * Copyright (c) 2000-2005 Silicon Graphics, Inc.
  * All Rights Reserved.
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write the Free Software Foundation,
- * Inc.,  51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include "libxfs_priv.h"
+#include "libxfs.h"
 #include "libxfs_io.h"
 #include "init.h"
 #include "xfs_fs.h"
@@ -242,7 +231,7 @@ xfs_flags2diflags2(
  * This was once shared with the kernel, but has diverged to the point
  * where it's no longer worth the hassle of maintaining common code.
  */
-int
+static int
 libxfs_ialloc(
 	xfs_trans_t	*tp,
 	xfs_inode_t	*pip,
@@ -251,7 +240,6 @@ libxfs_ialloc(
 	xfs_dev_t	rdev,
 	struct cred	*cr,
 	struct fsxattr	*fsx,
-	int		okalloc,
 	xfs_buf_t	**ialloc_context,
 	xfs_inode_t	**ipp)
 {
@@ -264,7 +252,7 @@ libxfs_ialloc(
 	 * Call the space management code to pick
 	 * the on-disk inode to be allocated.
 	 */
-	error = xfs_dialloc(tp, pip ? pip->i_ino : 0, mode, okalloc,
+	error = xfs_dialloc(tp, pip ? pip->i_ino : 0, mode,
 			    ialloc_context, &ino);
 	if (error != 0)
 		return error;
@@ -274,7 +262,8 @@ libxfs_ialloc(
 	}
 	ASSERT(*ialloc_context == NULL);
 
-	error = xfs_trans_iget(tp->t_mountp, tp, ino, 0, 0, &ip);
+	error = libxfs_iget(tp->t_mountp, tp, ino, 0, &ip,
+			&xfs_default_ifork_ops);
 	if (error != 0)
 		return error;
 	ASSERT(ip != NULL);
@@ -335,8 +324,8 @@ libxfs_ialloc(
 	case S_IFCHR:
 	case S_IFBLK:
 		ip->i_d.di_format = XFS_DINODE_FMT_DEV;
-		ip->i_df.if_u2.if_rdev = rdev;
 		flags |= XFS_ILOG_DEV;
+		VFS_I(ip)->i_rdev = rdev;
 		break;
 	case S_IFREG:
 	case S_IFDIR:
@@ -367,8 +356,8 @@ libxfs_ialloc(
 	case S_IFLNK:
 		ip->i_d.di_format = XFS_DINODE_FMT_EXTENTS;
 		ip->i_df.if_flags = XFS_IFEXTENTS;
-		ip->i_df.if_bytes = ip->i_df.if_real_bytes = 0;
-		ip->i_df.if_u1.if_extents = NULL;
+		ip->i_df.if_bytes = 0;
+		ip->i_df.if_u1.if_root = NULL;
 		break;
 	default:
 		ASSERT(0);
@@ -388,76 +377,12 @@ libxfs_ialloc(
 	/*
 	 * Log the new values stuffed into the inode.
 	 */
+	xfs_trans_ijoin(tp, ip, 0);
 	xfs_trans_log_inode(tp, ip, flags);
 	*ipp = ip;
 	return 0;
 }
 
-void
-libxfs_iprint(
-	xfs_inode_t		*ip)
-{
-	struct xfs_icdinode	*dip;
-	xfs_bmbt_rec_host_t	*ep;
-	xfs_extnum_t		i;
-	xfs_extnum_t		nextents;
-
-	printf("Inode %lx\n", (unsigned long)ip);
-	printf("    i_ino %llx\n", (unsigned long long)ip->i_ino);
-
-	if (ip->i_df.if_flags & XFS_IFEXTENTS)
-		printf("EXTENTS ");
-	printf("\n");
-	printf("    i_df.if_bytes %d\n", ip->i_df.if_bytes);
-	printf("    i_df.if_u1.if_extents/if_data %lx\n",
-		(unsigned long)ip->i_df.if_u1.if_extents);
-	if (ip->i_df.if_flags & XFS_IFEXTENTS) {
-		nextents = ip->i_df.if_bytes / (uint)sizeof(*ep);
-		for (ep = ip->i_df.if_u1.if_extents, i = 0; i < nextents;
-								i++, ep++) {
-			xfs_bmbt_irec_t rec;
-
-			xfs_bmbt_get_all(ep, &rec);
-			printf("\t%d: startoff %llu, startblock 0x%llx,"
-				" blockcount %llu, state %d\n",
-				i, (unsigned long long)rec.br_startoff,
-				(unsigned long long)rec.br_startblock,
-				(unsigned long long)rec.br_blockcount,
-				(int)rec.br_state);
-		}
-	}
-	printf("    i_df.if_broot %lx\n", (unsigned long)ip->i_df.if_broot);
-	printf("    i_df.if_broot_bytes %x\n", ip->i_df.if_broot_bytes);
-
-	dip = &ip->i_d;
-	printf("\nOn disk portion\n");
-	printf("    di_mode %o\n", VFS_I(ip)->i_mode);
-	printf("    di_version %x\n", (uint)dip->di_version);
-	switch (ip->i_d.di_format) {
-	case XFS_DINODE_FMT_LOCAL:
-		printf("    Inline inode\n");
-		break;
-	case XFS_DINODE_FMT_EXTENTS:
-		printf("    Extents inode\n");
-		break;
-	case XFS_DINODE_FMT_BTREE:
-		printf("    B-tree inode\n");
-		break;
-	default:
-		printf("    Other inode\n");
-		break;
-	}
-	printf("   di_nlink %x\n", VFS_I(ip)->i_nlink);
-	printf("   di_uid %d\n", dip->di_uid);
-	printf("   di_gid %d\n", dip->di_gid);
-	printf("   di_nextents %d\n", dip->di_nextents);
-	printf("   di_size %llu\n", (unsigned long long)dip->di_size);
-	printf("   di_gen %x\n", VFS_I(ip)->i_generation);
-	printf("   di_extsize %d\n", dip->di_extsize);
-	printf("   di_flags %x\n", dip->di_flags);
-	printf("   di_nblocks %llu\n", (unsigned long long)dip->di_nblocks);
-}
-
 /*
  * Writes a modified inode's changes out to the inode's on disk home.
  * Originally based on xfs_iflush_int() from xfs_inode.c in the kernel.
@@ -469,7 +394,6 @@ libxfs_iflush_int(xfs_inode_t *ip, xfs_buf_t *bp)
 	xfs_dinode_t		*dip;
 	xfs_mount_t		*mp;
 
-	ASSERT(XFS_BUF_FSPRIVATE(bp, void *) != NULL);
 	ASSERT(ip->i_d.di_format != XFS_DINODE_FMT_BTREE ||
 		ip->i_d.di_nextents > ip->i_df.if_ext_max);
 	ASSERT(ip->i_d.di_version > 1);
@@ -496,10 +420,8 @@ libxfs_iflush_int(xfs_inode_t *ip, xfs_buf_t *bp)
 	if (ip->i_d.di_version == 3)
 		VFS_I(ip)->i_version++;
 
-	/* Check the inline directory data. */
-	if (S_ISDIR(VFS_I(ip)->i_mode) &&
-	    ip->i_d.di_format == XFS_DINODE_FMT_LOCAL &&
-	    xfs_dir2_sf_verify(ip))
+	/* Check the inline fork data before we write out. */
+	if (!libxfs_inode_verify_forks(ip))
 		return -EFSCORRUPTED;
 
 	/*
@@ -560,8 +482,6 @@ libxfs_alloc_file_space(
 	xfs_filblks_t	datablocks;
 	xfs_filblks_t	allocated_fsb;
 	xfs_filblks_t	allocatesize_fsb;
-	xfs_fsblock_t	firstfsb;
-	struct xfs_defer_ops free_list;
 	xfs_bmbt_irec_t *imapp;
 	xfs_bmbt_irec_t imaps[1];
 	int		reccount;
@@ -599,19 +519,15 @@ libxfs_alloc_file_space(
 		}
 		xfs_trans_ijoin(tp, ip, 0);
 
-		xfs_defer_init(&free_list, &firstfsb);
 		error = xfs_bmapi_write(tp, ip, startoffset_fsb, allocatesize_fsb,
-				xfs_bmapi_flags, &firstfsb, 0, imapp,
-				&reccount, &free_list);
+				xfs_bmapi_flags, 0, imapp, &reccount);
 
 		if (error)
 			goto error0;
 
-		/* complete the transaction */
-		error = xfs_defer_finish(&tp, &free_list, ip);
-		if (error)
-			goto error0;
-
+		/*
+		 * Complete the transaction
+		 */
 		error = xfs_trans_commit(tp);
 		if (error)
 			break;
@@ -626,23 +542,10 @@ libxfs_alloc_file_space(
 	return error;
 
 error0:	/* Cancel bmap, cancel trans */
-	xfs_defer_cancel(&free_list);
 	xfs_trans_cancel(tp);
 	return error;
 }
 
-unsigned int
-libxfs_log2_roundup(unsigned int i)
-{
-	unsigned int	rval;
-
-	for (rval = 0; rval < NBBY * sizeof(i); rval++) {
-		if ((1 << rval) >= i)
-			break;
-	}
-	return rval;
-}
-
 /*
  * Wrapper around call to libxfs_ialloc. Takes care of committing and
  * allocating a new transaction as needed.
@@ -667,7 +570,7 @@ libxfs_inode_alloc(
 
 	ialloc_context = (xfs_buf_t *)0;
 	error = libxfs_ialloc(*tp, pip, mode, nlink, rdev, cr, fsx,
-			   1, &ialloc_context, &ip);
+			   &ialloc_context, &ip);
 	if (error) {
 		*ipp = NULL;
 		return error;
@@ -681,7 +584,7 @@ libxfs_inode_alloc(
 
 		xfs_trans_bhold(*tp, ialloc_context);
 
-		error = xfs_trans_roll(tp, NULL);
+		error = xfs_trans_roll(tp);
 		if (error) {
 			fprintf(stderr, _("%s: cannot duplicate transaction: %s\n"),
 				progname, strerror(error));
@@ -689,7 +592,7 @@ libxfs_inode_alloc(
 		}
 		xfs_trans_bjoin(*tp, ialloc_context);
 		error = libxfs_ialloc(*tp, pip, mode, nlink, rdev, cr,
-				   fsx, 1, &ialloc_context, &ip);
+				   fsx, &ialloc_context, &ip);
 		if (!ip)
 			error = -ENOSPC;
 		if (error)
@@ -746,13 +649,37 @@ cmn_err(int level, char *fmt, ...)
  */
 void
 xfs_verifier_error(
-	struct xfs_buf		*bp)
+	struct xfs_buf		*bp,
+	int			error,
+	xfs_failaddr_t		failaddr)
 {
-	xfs_alert(NULL, "Metadata %s detected at %s block 0x%llx/0x%x",
+	xfs_buf_ioerror(bp, error);
+
+	xfs_alert(NULL, "Metadata %s detected at %p, %s block 0x%llx/0x%x",
 		  bp->b_error == -EFSBADCRC ? "CRC error" : "corruption",
+		  failaddr ? failaddr : __return_address,
 		  bp->b_ops->name, bp->b_bn, BBTOB(bp->b_length));
 }
 
+/*
+ * Warnings for inode corruption problems.  Don't bother with the stack
+ * trace unless the error level is turned up high.
+ */
+void
+xfs_inode_verifier_error(
+	struct xfs_inode	*ip,
+	int			error,
+	const char		*name,
+	void			*buf,
+	size_t			bufsz,
+	xfs_failaddr_t		failaddr)
+{
+	xfs_alert(NULL, "Metadata %s detected at %p, inode 0x%llx %s",
+		  error == -EFSBADCRC ? "CRC error" : "corruption",
+		  failaddr ? failaddr : __return_address,
+		  ip->i_ino, name);
+}
+
 /*
  * This is called from I/O verifiers on v5 superblock filesystems. In the
  * kernel, it validates the metadata LSN parameter against the current LSN of
@@ -765,7 +692,7 @@ xfs_verifier_error(
  * repair can validate it against the state of the log.
  */
 xfs_lsn_t	libxfs_max_lsn = 0;
-pthread_mutex_t	libxfs_max_lsn_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t	libxfs_max_lsn_lock = PTHREAD_MUTEX_INITIALIZER;
 
 bool
 xfs_log_check_lsn(
@@ -825,3 +752,28 @@ libxfs_zero_extent(
 	return libxfs_device_zero(xfs_find_bdev_for_inode(ip), sector, size);
 }
 
+unsigned int
+hweight8(unsigned int w)
+{
+	unsigned int res = w - ((w >> 1) & 0x55);
+	res = (res & 0x33) + ((res >> 2) & 0x33);
+	return (res + (res >> 4)) & 0x0F;
+}
+
+unsigned int
+hweight32(unsigned int w)
+{
+	unsigned int res = w - ((w >> 1) & 0x55555555);
+	res = (res & 0x33333333) + ((res >> 2) & 0x33333333);
+	res = (res + (res >> 4)) & 0x0F0F0F0F;
+	res = res + (res >> 8);
+	return (res + (res >> 16)) & 0x000000FF;
+}
+
+unsigned int
+hweight64(__u64 w)
+{
+	return hweight32((unsigned int)w) +
+	       hweight32((unsigned int)(w >> 32));
+}
+
